# -*- coding: utf-8 -*-
"""
Module for plotting the data recorded by the monitor module during the 
tracking.

@author: Watanyu Foosang
@Date: 10/04/2020
"""

import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import h5py as hp

def plot_bunchdata(filename, bunch_number, dataset, option=None, x_var="time"):
    """
    Plot data recorded from a BunchMonitor.
    
    Parameters
    ----------
    filename : str
        Name of the HDF5 file that contains the data.
    bunch_number : int
        The bunch number whose data has been saved in the HDF5 file.
        This has to be identical to 'bunch_number' parameter in 'BunchMonitor' object.
    detaset : str {"current","emit","mean","std"}
        HDF5 file's dataset to be plotted.
    option : str, optional
        If dataset is "emit", "mean", or "std", the variable name to be plotted
        needs to be specified :
            for "emit", option = {"x","y","s"}
            for "mean" and "std", option = {"x","xp","y","yp","tau","delta"}    
    x_var : str, optional
        Variable to be plotted on the horizontal axis. The default is "time".

    """
    
    file = hp.File(filename, "r")
    
    group = "BunchData_{0}".format(bunch_number)  # Data group of the HDF5 file
    
    if dataset == "current":
        y_var = file[group][dataset][:]*1e3
        label = "current (mA)"
        
    elif dataset == "emit":
        # Specifying the axis
        # horizontal axis (x)   -> 0 
        # vertical axis (y)     -> 1 
        # longitudinal axis (s) -> 2 
                         
        emit_axis = {"x":0, "y":1, "s":2}
                         
        y_var = file[group][dataset][emit_axis[option]]*1e9
        
        if option == "x": label = "hor. emittance (nm.rad)"
        elif option == "y": label = "ver. emittance (nm.rad)"
        elif option == "s": label = "long. emittance (nm.rad)"
        
        
    elif dataset == "mean" or "std":
        # Specify the variable
        # x     -> 0 
        # xp    -> 1 
        # y     -> 2 
        # yp    -> 3 
        # tau   -> 4 
        # delta -> 5 
                           
        var_dict = {"x":0, "xp":1, "y":2, "yp":3, "tau":4, "delta":5}
        scale = [1e6, 1e6, 1e6, 1e6, 1e12, 1]
        label_list = ["x (um)", "x' ($\\mu$rad)", "y (um)", "y' ($\\mu$rad)",
                      "$\\tau$ (ps)", "$\\delta$"]
        
        y_var = file[group][dataset][var_dict[option]]*scale[var_dict[option]]
        label = label_list[var_dict[option]]
        
            
    plt.plot(file[group]["time"][:],y_var)
    plt.xlabel("number of turns")
    plt.ylabel(label)
    
    file.close()
            
def plot_phasespacedata(filename, bunch_number, x_var, y_var, turn, 
                        only_alive=True):
    """
    Plot data recorded from a PhaseSpaceMonitor.

    Parameters
    ----------
    filename : str
        Name of the HDF5 file that contains the data.
    bunch_number : int
        Number of the bunch whose data has been saved in the HDF5 file.
        This has to be identical to 'bunch_number' parameter in 
        'PhaseSpaceMonitor' object.
    x_var, y_var : str {"x", "xp", "y", "yp", "tau", "delta"}
        If dataset is "particles", the variables to be plotted on the 
        horizontal and the vertical axes need to be specified.
    turn : int
        Turn at which the data will be extracted.
    only_alive : bool, optional
        When only_alive is True, only alive particles are plotted and dead 
        particles will be discarded.
    """
    
    file = hp.File(filename, "r")
    
    group = "PhaseSpaceData_{0}".format(bunch_number)
    dataset = "particles"

    # Specify the parameter
    #     x     -> 0 
    #     xp    -> 1 
    #     y     -> 2 
    #     yp    -> 3 
    #     tau   -> 4 
    #     delta -> 5  
             
    var_dict = {"x":0, "xp":1, "y":2, "yp":3, "tau":4, "delta":5}
    scale = [1e3,1e3,1e3,1e3,1e12,1]
    label = ["x (mm)","x' (mrad)","y (mm)","y' (mrad)","$\\tau$ (ps)",
             "$\\delta$"]
    
    # find the index of "turn" in time array
    turn_index = np.where(file["PhaseSpaceData_0"]["time"][:]==turn) 
    
    if len(turn_index[0]) == 0:
        raise ValueError("Turn {0} is not found. Enter turn from {1}.".
                         format(turn, file[group]["time"][:]))     
    
    path = file[group][dataset]

    if only_alive is False:
        # format : sns.jointplot(x_axis, yaxis, kind)
        x_axis = path[:,var_dict[x_var],turn_index[0][0]]
        y_axis = path[:,var_dict[y_var],turn_index[0][0]]

    elif only_alive is True:
        alive_index = np.where(file[group]["alive"][:,turn_index])

        x_axis = path[alive_index[0],var_dict[x_var],turn_index[0][0]]
        y_axis = path[alive_index[0],var_dict[y_var],turn_index[0][0]]            
        
    sns.jointplot(x_axis*scale[var_dict[x_var]], 
                  y_axis*scale[var_dict[y_var]], kind="kde")
    
    plt.xlabel(label[var_dict[x_var]])
    plt.ylabel(label[var_dict[y_var]])
            
    file.close()

    
    
     
    
